%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% - Calculates the angular error between a contour and the vector field --%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% @parameters
%
% ellipse_points: contour points
% U: array 4D (lon,lat,depth,time) containing U current component
% V: array 4D (lon,lat,depth,time) containing V current component
% t: time
% depth: depth
% cyclonic: 1 if cyclonic, 0 if anticyclonic
%
% error: estimated contour mean error
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function error = calculate_vector_error(ellipse_points,U,V,t,depth,cyclonic)
    if(~cyclonic)
        ellipse_points = flipdim(ellipse_points,2);
    end
    ellipse_angles = get_angles_ellipse_tangent_vectors(ellipse_points);
    map_angles = get_map_angles(ellipse_points,U,V,t,depth);
    if(map_angles == -1)
        error = 1000000;
        return;
    end
    error = calculate_angles_diff(ellipse_angles,map_angles);
end

function ellipse_angles = get_angles_ellipse_tangent_vectors(ellipse_points)
    s = size(ellipse_points(1,:));
    ellipse_angles = zeros([s(1),s(2)-1]);
    for i=1:length(ellipse_points(1,:)) - 1
        ellipse_angles(i) = get_one_angle_ellipse_tangent_vector(ellipse_points,i);
    end
end

function angle = get_one_angle_ellipse_tangent_vector(points,i)
    angle = get_angle(points(1,i+1) - points(1,i),points(2,i+1) - points(2,i));
end

function map_angles = get_map_angles(ellipse_points,U,V,t,depth)
    s = size(ellipse_points(1,:));
    map_angles = zeros([s(1),s(2)-1]);
    for i=1:length(map_angles)
        map_angles(i) = get_one_map_angle(ellipse_points(1,i),ellipse_points(2,i),U,V,t,depth);
        if(map_angles(i) == -1)
            map_angles = -1;
            return;
        end
    end
end

function angle = get_one_map_angle(lon,lat,U,V,t,depth)
    lon_size = length(U(1,:,1,1));
    lat_size = length(U(:,1,1,1));
    if((round(lon) > lon_size) || (round(lat) > lat_size) || (round(lon) < 1) || (round(lat) < 1) || (U(round(lat),round(lon),depth,t) == 0) && (V(round(lat),round(lon),depth,t) == 0))
        angle = -1;
    else   
        angle = get_angle(U(round(lat),round(lon),1,t),V(round(lat),round(lon),1,t));
    end
end

function error = calculate_angles_diff(ellipse_angles,map_angles)
    error = 0;
    for i=1:length(map_angles)
        if(isnan(map_angles(i)))
            error = pi;
            continue;
        end
        error = error + get_angle_diff(ellipse_angles(i),map_angles(i));
        %error = error + get_angle_diff(ellipse_angles(i),map_angles(i))^3;
    end
    error = error / length(map_angles);
end