%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% - Check the three constraints ------------------------------------------%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% @parameters
%
% U: array 4D (lon,lat,depth,time) containing U current component
% V: array 4D (lon,lat,depth,time) containing V current component
% lon: eddy center longitude
% lat: eddy center latitud
% t: time
% depth: depth
% rotate_max_area: maximum area to check constraints
% max_angle: maximum angle (radians) allowed between current vectors
%
% res: 1 if the constraints are satisfied, 0 if not
% type: 1 if anticyclonic eddy, 2 if cyclonic eddy
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [res,type] = check_center_constraints(U,V,lon,lat,t,depth,rotate_max_area,max_angle)
    lon_size = length(U(1,:,1,1));
    lat_size = length(U(:,1,1,1));
    type = -1;
    res = 0;
    for i=2:rotate_max_area
        if((lon-i <= 0) || (lon+i > lon_size)|| (lat-i <= 0) || (lat+i > lat_size))
            continue;
        end
        res = check_vorticity(U,V,lon,lat,t,depth,i);
        if(res == 0)
           continue;
        end
        [res,type] = check_rotate_sense(U,V,lon,lat,t,depth,i,max_angle);
        if(res == 1)
           break;
        end
    end
end


function res = check_vorticity(U,V,lon,lat,t,depth,area)
    res = check_coherence(U,V,lon,lat,t,depth,area);
    if(res ~= 0)
        res = check_shape(U,V,lon,lat,t,depth,area);
    end
end


function res = check_coherence(U,V,lon,lat,t,depth,area)
    res = 1;
    if(U(lat+area,lon,depth,t) > 0)
        if(U(lat-area,lon,depth,t) >= 0) 
            res = 0;
            return;
        end
        if(V(lat,lon+area,depth,t) >= 0) 
            res = 0;
            return;
        end
        if(V(lat,lon-area,depth,t) <= 0)
            res = 0;
            return;
        end
    else
        if(U(lat-area,lon,depth,t) <= 0) 
            res = 0;
            return;
        end
        if(V(lat,lon+area,depth,t) <= 0) 
            res = 0;
            return;
        end
        if(V(lat,lon-area,depth,t) >= 0)
            res = 0;
            return;
        end
    end   
end


function res = check_shape(U,V,lon,lat,t,depth,area)
    nice_angle = 0;
    res = 0;
    alfa = get_angle(U(lat,lon-area,depth,t),V(lat,lon-area,depth,t));
    beta = get_angle(U(lat,lon+area,depth,t),V(lat,lon+area,depth,t));
    ganma = get_angle(U(lat-area,lon,depth,t),V(lat-area,lon,depth,t));
    delta = get_angle(U(lat+area,lon,depth,t),V(lat+area,lon,depth,t));
    if((alfa >= pi/4) && (alfa <= 3*pi/4) || (alfa >= 5*pi/4) && (alfa <= 7*pi/4))
        nice_angle = nice_angle + 1;
    end
    if((beta >= pi/4) && (beta <= 3*pi/4) || (beta >= 5*pi/4) && (beta <= 7*pi/4))
        nice_angle = nice_angle + 1;
    end  
    if((ganma >= 0) && (ganma <= pi/4) || (ganma >= 7*pi/4) && (ganma <= 0) || (ganma >= 3*pi/4) && (ganma <= 5*pi/4))
        nice_angle = nice_angle + 1;
    end 
    if((delta >= 0) && (delta <= pi/4) || (delta >= 7*pi/4) && (delta <= 0) || (delta >= 3*pi/4) && (delta <= 5*pi/4))
        nice_angle = nice_angle + 1;
    end 
    if(nice_angle >= 3)
        res = 1;
    end
end


function [res,type] = check_rotate_sense(U,V,lon,lat,t,depth,area,max_angle)
    type = -1;
    res = 1;
    %South border
    for j=lon-area:lon+area-1
        res = check_pair(U(lat-area,j,depth,t),V(lat-area,j,depth,t),U(lat-area,j+1,depth,t),V(lat-area,j+1,depth,t),max_angle);
        if(res == 0) 
            return;
        end
    end
    %East border
    for j=lat-area:lat+area-1
        res = check_pair(U(j,lon+area,depth,t),V(j,lon+area,depth,t),U(j+1,lon+area,depth,t),V(j+1,lon+area,depth,t),max_angle);
        if(res == 0) 
            return;
        end
    end
    %North border
    for j=lon+area:-1:lon-area+1
        res = check_pair(U(lat+area,j,depth,t),V(lat+area,j,depth,t),U(lat+area,j-1,depth,t),V(lat+area,j-1,depth,t),max_angle);
        if(res == 0)
            %disp('fin');
            return;
        end
    end  
    %West border
    for j=lat+area:-1:lat-area+1
        res = check_pair(U(j,lon-area,depth,t),V(j,lon-area,depth,t),U(j-1,lon-area,depth,t),V(j-1,lon-area,depth,t),max_angle);
        if(res == 0) 
            %disp('fin');
            return;
        end
    end
    if(U(lat+area,lon,depth,t) > 0) 
        type = 1;
    else
        type = 2;
    end
end

function res = check_pair(U1,V1,U2,V2,max_angle)
    MAX_INV_ANG_DIFF = 0.1745;
    res = 0;
    alfa = get_angle(U1,V1);
    beta = get_angle(U2,V2);
    if(beta > alfa)
        if(beta - alfa <= max_angle)
            res = 1;
        end
    else
        if((beta + 2*pi - alfa <= max_angle) || (alfa - beta <= MAX_INV_ANG_DIFF))
            res = 1;
        end
    end
end
