%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% - Check the three constraints ------------------------------------------%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% @parameters
%
% axis_1: array containing first axis coordinates (lon_1,lat_1,lon_2,lat_2)
% axis_2: array containing first axis coordinates (lon_3,lat_3,lon_4,lat_4)
% lon: eddy center longitude
% lat: eddy center latitud
% U: array 4D (lon,lat,depth,time) containing U current component
% V: array 4D (lon,lat,depth,time) containing V current component
% t: time
% depth: depth
%
% res: 1 if the constraints are satisfied, 0 if not
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function res = check_center_constraints_by_axis(axis_1,axis_2,angle,lon,lat,U,V,t,depth)
    lon_size = length(U(1,:,1,1));
    lat_size = length(U(:,1,1,1));
    res = 0;
    if(axis_in_range(axis_1,axis_2,lon_size,lat_size))
        res = check_vorticity_by_axis(axis_1,axis_2,angle,U,V,t,depth);
        if(res == 0)
           return;
        end
        res = check_rotate_sense_by_axis(axis_1,axis_2,lon,lat,angle,U,V,t,depth);
    end
end


function res = axis_in_range(axis_1,axis_2,lon_size,lat_size)
    res = 1;
    if((axis_1(1)<1)||(axis_1(3)<1)||(axis_1(2)<1)||(axis_1(4)<1)||(axis_2(1)<1)||(axis_2(3)<1)||(axis_2(2)<1)||(axis_2(4)<1))
        res = 0;
        return;
    end
    if((axis_1(1)>lon_size)||(axis_1(3)>lon_size)||(axis_1(2)>lat_size)||(axis_1(4)>lat_size)||(axis_2(1)>lon_size)||(axis_2(3)>lon_size)||(axis_2(2)>lat_size)||(axis_2(4)>lat_size))
        res = 0;
    end
end




function res = check_vorticity_by_axis(axis_1,axis_2,angle,U,V,t,depth)
    [alfa,beta,ganma,delta] = get_axis_map_angles(axis_1,axis_2,U,V,t,depth);
    alfa_check = clasify_angle(alfa,angle);
    beta_check = clasify_angle(beta,angle);
    ganma_check = clasify_angle(ganma,angle);
    delta_check = clasify_angle(delta,angle);
    res = 1;
    if(alfa_check == 2)
        if(beta_check ~= 4)
            res = 0;
            return;
        end
        if(ganma_check ~= 3)
            res = 0;
            return;
        end
        if(delta_check ~= 1)
            res = 0;
            return;
        end
    else
        if(alfa_check ~= 4)
            res = 0;
            return;
        else
            if(beta_check ~= 2)
                res = 0;
                return;
            end
            if(ganma_check ~= 1)
                res = 0;
                return;
            end
            if(delta_check ~= 3)
                res = 0;
                return;
            end    
        end
    end
end


function [alfa,beta,ganma,delta] = get_axis_map_angles(axis_1,axis_2,U,V,t,depth)
    axis_1 = round(axis_1);
    axis_2 = round(axis_2);
    alfa = get_angle(U(axis_1(2),axis_1(1),depth,t),V(axis_1(2),axis_1(1),depth,t));
    beta = get_angle(U(axis_1(4),axis_1(3),depth,t),V(axis_1(4),axis_1(3),depth,t));
    ganma = get_angle(U(axis_2(2),axis_2(1),depth,t),V(axis_2(2),axis_2(1),depth,t));
    delta = get_angle(U(axis_2(4),axis_2(3),depth,t),V(axis_2(4),axis_2(3),depth,t));
end


function alfa_check = clasify_angle(alfa,angle)
    alfa = alfa - angle;
    alfa = simplify_angle(alfa);
    if((alfa <= pi/4) || (alfa >= 7*pi/4))
        alfa_check = 1;
        return;
    end
    if((alfa <= 3*pi/4) && (alfa > pi/4))
        alfa_check = 2;
        return;
    end
    if((alfa <= 5*pi/4) && (alfa > 3*pi/4))
        alfa_check = 3;
        return;
    end
    alfa_check = 4;
end




function res = check_rotate_sense_by_axis(axis_1,axis_2,lon,lat,angle,U,V,t,depth)
    ellipse_points = get_ellipse_points(lon,lat,axis_1,axis_2,angle,1);
    %ellipse_points = flipdim(ellipse_points,2);
    res = check_rotate_sense_by_points(ellipse_points,U,V,t,depth);
end


function res = check_rotate_sense_by_points(ellipse_points,U,V,t,depth)
    lon_size = length(U(1,:,1,1));
    lat_size = length(U(:,1,1,1));
    sum_exceptions = 0;
    for i=1:length(ellipse_points(1,:))-1
        point_lat_1 = round(ellipse_points(2,i));
        point_lon_1 = round(ellipse_points(1,i));
        
        point_lat_2 = round(ellipse_points(2,i+1));
        point_lon_2 = round(ellipse_points(1,i+1));

        if((point_lon_1<1)||(point_lat_1<1)||(point_lon_1>lon_size)||(point_lat_1>lat_size)||(point_lon_2<1)||(point_lat_2<1)||(point_lon_2>lon_size)||(point_lat_2>lat_size))
            res = 0;
            return;
        end
        [res,exception] = check_pair(U(point_lat_1,point_lon_1,depth,t),V(point_lat_1,point_lon_1,depth,t),U(point_lat_2,point_lon_2,depth,t),V(point_lat_2,point_lon_2,depth,t));
        if(res == 0)
            return;
        end
        if(exception == 1)
            sum_exceptions = sum_exceptions + 1;
        else
            sum_exceptions = 0;
        end
        if(sum_exceptions == 3)
            res = 0;
            return;
        end
    end
end
  

function [res,exception] = check_pair(U1,V1,U2,V2)
    MAX_ANG_DIFF = pi/3;
    MAX_INV_ANG_DIFF = 0.1745;
    res = 0;
    exception = 0;
    alfa = get_angle(U1,V1);
    beta = get_angle(U2,V2);
    if(beta >= alfa)
        if(beta - alfa <= pi/3)
            res = 1;
        end
    else
        if((beta + 2*pi - alfa <= MAX_ANG_DIFF))
            res = 1;
        end
        if(alfa - beta <= MAX_INV_ANG_DIFF)
            res = 1;
            exception = 1;
        end
    end
end


