%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% - Compares curl detection with manual detection----------------------%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% @parameters
%
% data_1: eddy manual detection features (obtained with eddy_manual_selection)
% data_2: eddy curl detection features (obtained with eddy_shape_curl detection)
% vec_lon: array 1D longitudes
% vec_lat: array 1D latitudes
% U: array 4D (lon,lat,depth,time) containing U current component
% V: array 4D (lon,lat,depth,time) containing V current component
% t: time
% depth: depth
%
% match: number of matched eddies in both eddy sets
% found_ratio: percentage of found eddies
% excess_ratio: percentage of detection excess
% manual_error: mean angular error between manual detection eddies and vector field
% automatic_error: mean angular error between curl detection eddies and vector field
% contour_error: difference between automatic_error and manual_error
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [match,found_ratio,excess_ratio,manual_error,automatic_error,contour_error] = compare_eddy_curl(data_1,data_2,vec_lon,vec_lat,U,V,t,depth)
    match = 0;
    contour_error = 0;
    manual_error = 0;
    automatic_error = 0;
    for i=1:length(data_1)
        for j=1:length(data_2)
            if(equal_center_position(vec_lon,vec_lat,data_1(i),data_2(j),depth))
                match = match + 1;
                turned_data_1 = turn(vec_lon,vec_lat,data_1(i),depth);
                ellipse_points = get_ellipse_points_by_rads(turned_data_1.c_lon,turned_data_1.c_lat,turned_data_1.r1,turned_data_1.r2,turned_data_1.angle,0);
                contour = turn_contour(vec_lon,vec_lat,data_2(j).limit);
                [diff_error,error_1,error_2] =  compare_eddy_contour(ellipse_points,contour,data_2(j).cyclonic,U,V,t,depth);
                contour_error = contour_error + diff_error;
                manual_error = manual_error + error_1;
                automatic_error = automatic_error + error_2;
            end
        end
    end
    found_ratio = match*100/length(data_1);
    excess_ratio = (length(data_2) - match)*100/length(data_2);
    contour_error = contour_error / match;
    manual_error = manual_error / match;
    automatic_error = automatic_error / match;
end


function [error,error_manual,error_curl] = compare_eddy_contour(contour_1,contour_2,cyclonic,U,V,t,depth)
    error_manual = calculate_vector_error(contour_1,U,V,t,depth,cyclonic);
    error_curl_1 = calculate_vector_error(contour_2,U,V,t,depth,1);
    error_curl_2 = calculate_vector_error(contour_2,U,V,t,depth,0);
    if(error_curl_1 < error_curl_2)
        error_curl = error_curl_1;
    else
        error_curl = error_curl_2;
    end
    error = error_curl - error_manual;
end


function turned_data = turn(vec_lon,vec_lat,data,depth)
    step_lon = vec_lon(2) - vec_lon(1);
    step_lat = vec_lat(2) - vec_lat(1);
    turned_data.c_lon = (data.c_lon(depth) - vec_lon(1))/step_lon+1;
    turned_data.c_lat = (data.c_lat(depth) - vec_lat(1))/step_lat+1;
    turned_data.r1 = norm([abs(data.r1(depth)*cos(data.angle(depth)))/step_lon abs(data.r1(depth)*sin(data.angle(depth)))/step_lat]);
    turned_data.r2 = norm([abs(data.r2(depth)*cos(data.angle(depth)+pi/2))/step_lon abs(data.r2(depth)*sin(data.angle(depth)+pi/2))/step_lat]);
    turned_data.angle = data.angle(depth);
end

function contour = turn_contour(vec_lon,vec_lat,contour)
    step_lon = vec_lon(2) - vec_lon(1);
    step_lat = vec_lat(2) - vec_lat(1);
    for i=1:length(contour(1,:))
        contour(1,i) = (contour(1,i) - vec_lon(1))/step_lon+1;
        contour(2,i) = (contour(2,i) - vec_lat(1))/step_lat+1;
    end
end



function res = equal_center_position(vec_lon,vec_lat,data_1,data_2,depth)
    step_lon = vec_lon(2) - vec_lon(1);
    step_lat = vec_lat(2) - vec_lat(1);
    res = 0;
    if((data_2.c_lon < data_1.c_lon(depth) + step_lon) && (data_2.c_lon > data_1.c_lon(depth) - step_lon) && (data_2.c_lat < data_1.c_lat(depth) + step_lat) && (data_2.c_lat > data_1.c_lat(depth) - step_lat))
        res = 1;
    end   
end


function ellipse_points = get_ellipse_points_by_rads(lon,lat,dist_axis_1,dist_axis_2,angle,steps_are_important)
    if(steps_are_important)
        steps = 4*(round(dist_axis_1)+round(dist_axis_2))-4;
        if(steps < 8)
            steps = 8;
        end
    else
        steps = 50;
    end
    ellipse_points = zeros(2,steps);
    [ellipse_points(1,:),ellipse_points(2,:)] = calculate_ellipse(lon,lat,dist_axis_1,dist_axis_2,-180/pi*angle,steps);
end