%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% - Compares curl detection with manual detection----------------------%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% @parameters
%
% data_1: eddy manual detection features (obtained with eddy_manual_selection)
% data_2: eddy curl detection features (obtained with eddy_shape_curl detection)
% vec_lon: array 1D longitudes
% vec_lat: array 1D latitudes
% depth: depth
%
% match: number of matched eddies in both eddy sets
% radio_props_error: proportions between first radious and second radious mean error
% raw_angle: mean angular error
% perceptual_angle: mean angular error considering radious proportions
% size_error: mean between radious 1 and radious 2
% global_error: mean of radio_props_error, perceptual_angle and size_error
% found_ratio: percentage of found eddies
% excess_ratio: percentage of detection excess
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [match,radio_props_error,raw_angle_error,found_ratio,excess_ratio,perceptual_angle_error,size_error,global_error] = compare_eddy_ellipse(vec_lon,vec_lat,data_1,data_2,depth)
    match = 0;
    raw_angle_error = 0;
    radio_props_error = 0;
    radio_props_1 = 0;
    radio_props_2 = 0;
    perceptual_angle_error = 0;
    size_error = 0;
    for i=1:length(data_1)
        for j=1:length(data_2)
            if(equal_center_position(vec_lon,vec_lat,data_1(i),data_2(j),depth))
                match = match + 1;
                if(data_1(i).r1(depth) > data_1(i).r2(depth))
                    angle_1 = data_1(i).angle(depth);
                    R1_1 = data_1(i).r1(depth);
                    R2_1 = data_1(i).r2(depth);
                else
                    angle_1 = data_1(i).angle(depth) + pi/2;
                    R1_1 = data_1(i).r2(depth);
                    R2_1 = data_1(i).r1(depth);
                end
                if(data_2(j).r1 > data_2(j).r2)
                    angle_2 = data_2(j).angle;
                    R1_2 = data_2(j).r1;
                    R2_2 = data_2(j).r2;
                else
                    angle_2 = data_2(j).angle + pi/2;
                    R1_2 = data_2(j).r2;
                    R2_2 = data_2(j).r1;
                end
                raw_angle_error = raw_angle_error + axial_diff(angle_1,angle_2);
                radio_props_error = radio_props_error + abs(R1_1/R2_1 - R1_2/R2_2);
                radio_props_1 = radio_props_1 + R1_1/R2_1;
                radio_props_2 = radio_props_2 + R1_2/R2_2;
                mean_radio_1 = (R1_1 + R2_1)/2;
                mean_radio_2 = (R1_2 + R2_2)/2;
                size_error = size_error + abs(mean_radio_1 - mean_radio_2);
                mean_radio_props = (R1_1/R2_1+R1_2/R2_2)/2;
                if(mean_radio_props >= 1.3)
                    perceptual_angle_error = perceptual_angle_error + axial_diff(angle_1,angle_2);
                else
                    perceptual_angle_error = perceptual_angle_error + axial_diff(angle_1,angle_2)*(mean_radio_props-1);
                end
            end
        end
    end
    size_error = size_error / match;
    raw_angle_error = (raw_angle_error / match)*100/(pi/2);
    radio_props_error = radio_props_error / match;
    radio_props_1 = radio_props_1 / match;
    radio_props_2 = radio_props_2 / match;
    perceptual_angle_error = perceptual_angle_error / match;
    found_ratio = match*100/length(data_1);
    excess_ratio = (length(data_2) - match)*100/length(data_2);
    
    
    plot([-5 5],[0 0],'k','linewidth',2);
    hold on;
    plot([0 0],[-5 5],'k','linewidth',2);
    
    plot([-3.75 -3.75],[0 found_ratio*5/100],'linewidth',4);
    plot([-2.5 -2.5],[0 excess_ratio*5/100],'r','linewidth',4)
    plot([-1.25 -1.25],[0 10*match/(length(data_1)+length(data_2))],'g','linewidth',4)
    
    plot([0.5 4.5],[2.5 2.5],'k','linewidth',2);
    plot([2.5 2.5],[0.5 4.5],'k','linewidth',2);
    [X Y] = calculate_ellipse(2.5,2.5,1,radio_props_1,0);
    h1=plot(X,Y,'linewidth',3);
    [X Y] = calculate_ellipse(2.5,2.5,1,radio_props_2,0);
    h2=plot(X,Y,'r','linewidth',3);
    legend([h1,h2],'Manual','Automatic');
    
    plot([-4.5 -0.5],[-2.5 -2.5],'k','linewidth',2);
    plot([-2.5 -2.5],[-4.5 -0.5],'k','linewidth',2);  
    plot([-2.5 -1],[-2.5 -2.5],'linewidth',3);
    
    point = [(1.5)*cos(perceptual_angle_error)-2.5 (1.5)*sin(perceptual_angle_error)-2.5];
    plot([-2.5 point(1)],[-2.5 point(2)],'r','linewidth',3);
    
    plot([0.5 4.5],[-2.5 -2.5],'k','linewidth',2);
    plot([2.5 2.5],[-4.5 -0.5],'k','linewidth',2); 
    
    [X Y] = calculate_ellipse(2.5,-2.5,1,radio_props_1,0);
    plot(X,Y,'linewidth',3);
    [X Y] = calculate_ellipse(2.5,-2.5,1,radio_props_2,-perceptual_angle_error*180/pi);
    plot(X,Y,'r','linewidth',3);
    
   text(-4.6,1,'Found %','FontSize',7,'FontWeight','bold');
   text(-3.4,1,'Excess %','FontSize',7,'FontWeight','bold');
   text(-2.3,1,'Matching %','FontSize',7,'FontWeight','bold');
   text(-4.7,-2,'Perceptual Angle Error','FontSize',8,'FontWeight','bold');
   text(3.5,1,'Proportions Error','FontSize',8,'FontWeight','bold');
   text(3.5,-1,'Global Error','FontSize',8,'FontWeight','bold');
   
   match = 100*match/((length(data_1)+length(data_2))/2);
   global_error = (size_error + radio_props_error + perceptual_angle_error)/3;
   perceptual_angle_error = perceptual_angle_error*100/(pi/2);
   
   axis equal;
end

function res = equal_center_position(vec_lon,vec_lat,data_1,data_2,depth)
    step_lon = vec_lon(2) - vec_lon(1);
    step_lat = vec_lat(2) - vec_lat(1);
    res = 0;
    if((data_2.c_lon < data_1.c_lon(depth) + step_lon) && (data_2.c_lon > data_1.c_lon(depth) - step_lon) && (data_2.c_lat < data_1.c_lat(depth) + step_lat) && (data_2.c_lat > data_1.c_lat(depth) - step_lat))
        res = 1;
    end   
end

function diff = axial_diff(angle_1,angle_2)
    diff_1 = abs(angle_1 - angle_2);
    diff_2 = pi - angle_2 + angle_1;
    diff_3 = pi - angle_1 + angle_2;
    diff = min([diff_1 diff_2 diff_3]);
end