%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% - Detects eddy centers in a current map -------------------------------%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% @parameters
%
% U: array 4D (lon,lat,depth,time) containing U current component
% V: array 4D (lon,lat,depth,time) containing V current component
% rot_map: array 2D containing curl surface
% t: time
% depth: depth
% rotate_max_area: maximum area to check constraints
% max_angle: maximum angle allowed between eddy limit vectors
%
% real_centers_min: array 2D anticyclonic eddy centers
% rot_centers_min: array 2D eddy curl local min 
% real_centers_max: array 2D cyclonic eddy centers
% rot_centers_max: array 2D eddy curl local max
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [real_centers_min,rot_centers_min,real_centers_max,rot_centers_max] = eddy_center_detection(U,V,rot_map,t,depth,rotate_max_area,max_angle)
    
    lon_size = length(U(1,:,1,1));
    lat_size = length(U(:,1,1,1));

    local_min = imregionalmin(rot_map);
    local_max = imregionalmax(rot_map);
    local_min = img2matrix(local_min);
    local_max = img2matrix(local_max);
    [fila_min,columna_min] = find(local_min == 1);
    [fila_max,columna_max] = find(local_max == 1);

    real_centers_min = [];
    rot_centers_min = [];
    rot_values_min = [];
    %Anticiclonico
    for i=1:length(fila_min)
        [~,curr_lon,curr_lat] = get_current_min(columna_min(i),fila_min(i),U,V,t,depth);
        [res,type] = check_center_constraints(U,V,curr_lon,curr_lat,t,depth,rotate_max_area,max_angle);
        if((res == 0) || (res == 1) && (type == 2))
            continue;
        end
        real_centers_min = [real_centers_min,[curr_lon;curr_lat]];
        rot_centers_min = [rot_centers_min,[columna_min(i);fila_min(i)]];
        rot_values_min = [rot_values_min,rot_map(fila_min(i),columna_min(i))];
    end
    if(~isempty(real_centers_min))
        [real_centers_min,rot_centers_min] = unique_centers(real_centers_min,rot_centers_min,rot_values_min,lon_size,lat_size,'min');
    end

    real_centers_max = [];
    rot_centers_max = [];
    rot_values_max = [];
    %Ciclonico
    for i=1:length(fila_max)
        [~,curr_lon,curr_lat] = get_current_min(columna_max(i),fila_max(i),U,V,t,depth);
        [res,type] = check_center_constraints(U,V,curr_lon,curr_lat,t,depth,rotate_max_area,max_angle);
        if((res == 0) || (res == 1) && (type == 1))
            continue;
        end
        real_centers_max = [real_centers_max,[curr_lon;curr_lat]];
        rot_centers_max = [rot_centers_max,[columna_max(i);fila_max(i)]];
        rot_values_max = [rot_values_max,rot_map(fila_max(i),columna_max(i))];
    end
    if(~isempty(real_centers_max))
        [real_centers_max,rot_centers_max] = unique_centers(real_centers_max,rot_centers_max,rot_values_max,lon_size,lat_size,'max');
    end
end



function [real_centers_unique,rot_centers_unique] = unique_centers(real_centers,rot_centers,rot_values,lon_size,lat_size,max_min)
    real_centers_unique = [];
    rot_centers_unique = [];
    for i=1:length(real_centers(1,:))
        rot_centers_to_check = [];
        rot_values_to_check = [];
        if(isempty(find(ismember(real_centers_unique',[real_centers(1,i),real_centers(2,i)],'rows'),1)))
            real_centers_unique = [real_centers_unique,[real_centers(:,i)]];
            for j=1:length(real_centers(1,:))
                if((real_centers_unique(1,length(real_centers_unique(1,:))) == real_centers(1,j)) && (real_centers_unique(2,length(real_centers_unique(1,:))) == real_centers(2,j)))
                    rot_centers_to_check = [rot_centers_to_check,[rot_centers(:,j)]];
                    rot_values_to_check = [rot_values_to_check,rot_values(j)];
                end
            end
            chosen_rot_center = choose_rot_center(rot_centers_to_check,real_centers(:,i),rot_values_to_check,lon_size,lat_size,max_min);
            rot_centers_unique = [rot_centers_unique,chosen_rot_center];
        end
    end
end

function chosen_rot_center = choose_rot_center(rot_centers_to_check,real_center,rot_values_to_check,lon_size,lat_size,max_min)
    
    DIST_FACTOR = 100;
    ROT_FACTOR = 40;

    distances = get_distances_to_real_center(rot_centers_to_check,real_center);
    size_factor = (lon_size + lat_size)/2;
    if(strcmp(max_min,'max'))
       val = max(rot_values_to_check); 
    else
       val = min(rot_values_to_check);
    end
    for i=1:length(distances)
        distances(i) = DIST_FACTOR*distances(i)/size_factor + ROT_FACTOR*abs(val-rot_values_to_check(i));
    end
    [~,index] = min(distances);
    chosen_rot_center = rot_centers_to_check(:,index);
end

function distances = get_distances_to_real_center(rot_centers,real_center)
    distances = zeros(1,length(rot_centers(1,:)));
    for i=1:length(rot_centers(1,:))
        distances(i) = norm(real_center' - rot_centers(:,i)');
    end
end


