%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% - Shows a current map and plots the eddy contours (ellipses) -----------%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% @parameters
%
% vec_lon: array 1D containing longitudes
% vec_lat: array 1D containing latitudes
% U: array 4D (lon,lat,depth,time) containing U current component
% V: array 4D (lon,lat,depth,time) containing V current component
% t: time
% depth: depth
% rotate_max_area: maximum area to check constraints
% max_angle: maximum angle (radians) allowed between current vectors
%
% automatic_info_eddy_array: array of structs containing eddy features
% error: estimated contours mean error
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [automatic_info_eddy_array,error] = eddy_shape_ellipse_detection(vec_lon,vec_lat,U,V,t,depth,rotate_max_area,max_angle)
    automatic_info_eddy = struct('c_lon',-1,'c_lat',-1,'r1',-1,'r2',-1,'angle',-1,'cyclonic',0);
    automatic_info_eddy_array = [];
    
    hold off;
    draw_map(vec_lon,vec_lat,U,V,depth,t);
    axis equal;
    
    [U,V]=delete_nan(U,V,t,depth);
    rot_map = curl(vec_lon, vec_lat, U(:,:,depth,t), V(:,:,depth,t));

    [real_centers_min,~,real_centers_max,~] = eddy_center_detection(U,V,rot_map,t,depth,rotate_max_area,max_angle);
    error = 0;
    if(~isempty(real_centers_min))
        for i=1:length(real_centers_min(1,:))
            automatic_info_eddy.cyclonic = 0;
            automatic_info_eddy.c_lon = vec_lon(real_centers_min(1,i));
            automatic_info_eddy.c_lat = vec_lat(real_centers_min(2,i));
            [axis_1,axis_2,angle] = find_eddy_orientation(real_centers_min(1,i),real_centers_min(2,i),U,V,t,depth,rotate_max_area);
            [axis_1,axis_2,~] = estimate_eddy_shape(axis_1,axis_2,angle,real_centers_min(1,i),real_centers_min(2,i),U,V,t,depth,0);
            [axis_1,axis_2,error_min] = estimate_eddy_size(axis_1,axis_2,angle,real_centers_min(1,i),real_centers_min(2,i),U,V,t,depth,0);
            error = error + error_min;
            automatic_info_eddy.angle = angle;
            [axis_1,axis_2] = change_reference_system(axis_1,axis_2,vec_lon,vec_lat);
            [automatic_info_eddy.r1,automatic_info_eddy.r2] = calculate_radious(axis_1,axis_2);
            automatic_info_eddy_array = [automatic_info_eddy_array,automatic_info_eddy];
            ellipse_points = get_ellipse_points(vec_lon(real_centers_min(1,i)),vec_lat(real_centers_min(2,i)),axis_1,axis_2,angle,0);
            plot(ellipse_points(1,:),ellipse_points(2,:),'r','linewidth',2);
        end
    end
    if(~isempty(real_centers_max))
        for i=1:length(real_centers_max(1,:))
            automatic_info_eddy.cyclonic = 1;
            automatic_info_eddy.c_lon = vec_lon(real_centers_max(1,i));
            automatic_info_eddy.c_lat = vec_lat(real_centers_max(2,i));
            [axis_1,axis_2,angle] = find_eddy_orientation(real_centers_max(1,i),real_centers_max(2,i),U,V,t,depth,rotate_max_area);
            [axis_1,axis_2,~] = estimate_eddy_shape(axis_1,axis_2,angle,real_centers_max(1,i),real_centers_max(2,i),U,V,t,depth,1);
            [axis_1,axis_2,error_max] = estimate_eddy_size(axis_1,axis_2,angle,real_centers_max(1,i),real_centers_max(2,i),U,V,t,depth,1);
            error = error + error_max;
            automatic_info_eddy.angle = angle;
            [axis_1,axis_2] = change_reference_system(axis_1,axis_2,vec_lon,vec_lat);
            [automatic_info_eddy.r1,automatic_info_eddy.r2] = calculate_radious(axis_1,axis_2);
            automatic_info_eddy_array = [automatic_info_eddy_array,automatic_info_eddy];
            ellipse_points = get_ellipse_points(vec_lon(real_centers_max(1,i)),vec_lat(real_centers_max(2,i)),axis_1,axis_2,angle,0);
            plot(ellipse_points(1,:),ellipse_points(2,:),'b','linewidth',2);
        end
    end
    error = error / length(automatic_info_eddy_array);
    save(strcat(strcat(strcat(strcat('automatic_ellipse_',num2str(depth)),'_'),num2str(t)),'.mat'),'automatic_info_eddy_array');
end


function [axis_1,axis_2] = change_reference_system(axis_1,axis_2,vec_lon,vec_lat)
    step_lon = vec_lon(2) - vec_lon(1);
    step_lat = vec_lat(2) - vec_lat(1);
    axis_1(1) = vec_lon(floor(axis_1(1))) + step_lon * (axis_1(1) - floor(axis_1(1)));
    axis_1(3) = vec_lon(floor(axis_1(3))) + step_lon * (axis_1(3) - floor(axis_1(3)));
    axis_2(1) = vec_lon(floor(axis_2(1))) + step_lon * (axis_2(1) - floor(axis_2(1)));
    axis_2(3) = vec_lon(floor(axis_2(3))) + step_lon * (axis_2(3) - floor(axis_2(3)));
    axis_1(2) = vec_lat(floor(axis_1(2))) + step_lat * (axis_1(2) - floor(axis_1(2)));
    axis_1(4) = vec_lat(floor(axis_1(4))) + step_lat * (axis_1(4) - floor(axis_1(4)));
    axis_2(2) = vec_lat(floor(axis_2(2))) + step_lat * (axis_2(2) - floor(axis_2(2)));
    axis_2(4) = vec_lat(floor(axis_2(4))) + step_lat * (axis_2(4) - floor(axis_2(4)));
end


function [r1,r2] = calculate_radious(axis_1,axis_2)
    r1 = sqrt((axis_1(1)-axis_1(3))*(axis_1(1)-axis_1(3))+(axis_1(2)-axis_1(4))*(axis_1(2)-axis_1(4)))/2;
    r2 = sqrt((axis_2(1)-axis_2(3))*(axis_2(1)-axis_2(3))+(axis_2(2)-axis_2(4))*(axis_2(2)-axis_2(4)))/2;
end