%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% - Estimates eddy shape----------------%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% @parameters
%
% axis_1: array 1D containing axis 1 coordinates (lon_1,lat_1,lon_2,lat_2) (matrix coordinate system)
% axis_2: array 1D containing axis 2 coordinates (lon_3,lat_3,lon_4,lat_4) (matrix coordinate system)
% angle: eddy angle (ellipse angle)
% lon: eddy center longitude (matrix coordinate system)
% lat: eddy center latitude (matrix coordinate system)
% U: array 4D (lon,lat,depth,time) containing U current component
% V: array 4D (lon,lat,depth,time) containing V current component
% t: time
% depth: depth
% cyclonic: 1 if cyclonic, 0 if not
%
% axis_1: array 1D containing new axis 1 coordinates (lon_1,lat_1,lon_2,lat_2) (matrix coordinate system)
% axis_2: array 1D containing new axis 2 coordinates (lon_3,lat_3,lon_4,lat_4) (matrix coordinate system)
% old_error: result error
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [axis_1,axis_2,old_error] = estimate_eddy_shape(axis_1,axis_2,angle,lon,lat,U,V,t,depth,cyclonic)
    ellipse_points = get_ellipse_points(lon,lat,axis_1,axis_2,angle,1);
    old_error = calculate_vector_error(ellipse_points,U,V,t,depth,cyclonic);
    [new_axis_1,new_axis_2,error,target_axis] = get_better_axis_to_reduce(lon,lat,axis_1,axis_2,angle,U,V,t,depth,cyclonic);
    while(error < old_error)
        axis_1 = new_axis_1;
        axis_2 = new_axis_2;
        old_error = error;
        [new_axis_1,new_axis_2] = reduce_axis(lon,lat,axis_1,axis_2,target_axis);
        ellipse_points = get_ellipse_points(lon,lat,new_axis_1,new_axis_2,angle,1);
        error = calculate_vector_error(ellipse_points,U,V,t,depth,cyclonic);
    end
end


function [axis_1,axis_2,error,target_axis] = get_better_axis_to_reduce(lon,lat,axis_1,axis_2,angle,U,V,t,depth,cyclonic)
    [new_axis_1_a,new_axis_2_a] = reduce_axis(lon,lat,axis_1,axis_2,0);
    ellipse_points = get_ellipse_points(lon,lat,new_axis_1_a,new_axis_2_a,angle,1);
    error_a = calculate_vector_error(ellipse_points,U,V,t,depth,cyclonic);    

    [new_axis_1_b,new_axis_2_b] = reduce_axis(lon,lat,axis_1,axis_2,1);
    ellipse_points = get_ellipse_points(lon,lat,new_axis_1_b,new_axis_2_b,angle,1);
    error_b = calculate_vector_error(ellipse_points,U,V,t,depth,cyclonic);   
    
    if(error_a < error_b)
        axis_1 = new_axis_1_a;
        axis_2 = new_axis_2_a;
        error = error_a;
        target_axis = 0;
    else
        axis_1 = new_axis_1_b;
        axis_2 = new_axis_2_b;
        error = error_b;
        target_axis = 1;
    end
end


function [new_axis_1,new_axis_2] = reduce_axis(lon,lat,axis_1,axis_2,target_axis)
    if(target_axis == 0)
        dist_lon = (abs(lon - axis_1(1)) + abs(lon - axis_1(3))) / 2;
        dist_lat = (abs(lat - axis_1(2)) + abs(lat - axis_1(4))) / 2;
        if(axis_1(1) < lon) new_axis_1(1) = axis_1(1) + dist_lon / 10; else new_axis_1(1) = axis_1(1) - dist_lon / 10; end
        if(axis_1(3) < lon) new_axis_1(3) = axis_1(3) + dist_lon / 10; else new_axis_1(3) = axis_1(3) - dist_lon / 10; end
        if(axis_1(2) < lat) new_axis_1(2) = axis_1(2) + dist_lat / 10; else new_axis_1(2) = axis_1(2) - dist_lat / 10; end
        if(axis_1(4) < lat) new_axis_1(4) = axis_1(4) + dist_lat / 10; else new_axis_1(4) = axis_1(4) - dist_lat / 10; end
        new_axis_2 = axis_2;
    else
        dist_lon = (abs(lon - axis_2(1)) + abs(lon - axis_2(3))) / 2;
        dist_lat = (abs(lat - axis_2(2)) + abs(lat - axis_2(4))) / 2;
        if(axis_2(1) < lon) new_axis_2(1) = axis_2(1) + dist_lon / 10; else new_axis_2(1) = axis_2(1) - dist_lon / 10; end
        if(axis_2(3) < lon) new_axis_2(3) = axis_2(3) + dist_lon / 10; else new_axis_2(3) = axis_2(3) - dist_lon / 10; end
        if(axis_2(2) < lat) new_axis_2(2) = axis_2(2) + dist_lat / 10; else new_axis_2(2) = axis_2(2) - dist_lat / 10; end
        if(axis_2(4) < lat) new_axis_2(4) = axis_2(4) + dist_lat / 10; else new_axis_2(4) = axis_2(4) - dist_lat / 10; end
        new_axis_1 = axis_1;
    end
end