%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% - Finds eddy orientation (angle)----------------%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% @parameters
%
% lon: eddy center longitude (matrix coordinate system)
% lat: eddy center latitude (matrix coordinate system)
% U: array 4D (lon,lat,depth,time) containing U current component
% V: array 4D (lon,lat,depth,time) containing V current component
% t: time
% depth: depth
% max_area: maximum area to check constraints
%
% axis_1: array 1D containing new axis 1 coordinates (lon_1,lat_1,lon_2,lat_2) (matrix coordinate system)
% axis_2: array 1D containing new axis 2 coordinates (lon_3,lat_3,lon_4,lat_4) (matrix coordinate system)
% angle: new eddy angle
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [axis_1,axis_2,angle] = find_eddy_orientation(center_lon,center_lat,U,V,t,depth,max_area)  
    lon_size = length(U(1,:,1,1));
    lat_size = length(U(:,1,1,1));
    min_diff_angle_dist = [10000000,0,0];
    for i=1:max_area
        for j=1:2*i+1
            [axis_1,axis_2] = get_axis(center_lon,center_lat,i,j);
            if(axis_in_range(axis_1,axis_2,lon_size,lat_size))
                vec_angle_1 = get_angle(U(axis_1(2),axis_1(1),depth,t),V(axis_1(2),axis_1(1),depth,t));
                vec_angle_2 = get_angle(U(axis_1(4),axis_1(3),depth,t),V(axis_1(4),axis_1(3),depth,t));
                vec_angle_3 = get_angle(U(axis_2(2),axis_2(1),depth,t),V(axis_2(2),axis_2(1),depth,t));
                vec_angle_4 = get_angle(U(axis_2(4),axis_2(3),depth,t),V(axis_2(4),axis_2(3),depth,t));
                if(isnan(vec_angle_1) || isnan(vec_angle_2) || isnan(vec_angle_3) || isnan(vec_angle_4))
                    break;
                end
                ort_diff_1 = ortogonal_diff(vec_angle_1,axis_1(1),axis_1(2),center_lon,center_lat);
                ort_diff_2 = ortogonal_diff(vec_angle_2,axis_1(3),axis_1(4),center_lon,center_lat);
                ort_diff_3 = ortogonal_diff(vec_angle_3,axis_2(1),axis_2(2),center_lon,center_lat);
                ort_diff_4 = ortogonal_diff(vec_angle_4,axis_2(3),axis_2(4),center_lon,center_lat);
                mean = (ort_diff_1^2 + ort_diff_2^2 + ort_diff_3^2 + ort_diff_4^2) / 4;
                if(mean < min_diff_angle_dist(1))
                    min_diff_angle_dist(1) = mean;
                    min_diff_angle_dist(2) = i;
                    min_diff_angle_dist(3) = j;
                end
            end
        end
    end
    [axis_1,axis_2] = get_axis(center_lon,center_lat,min_diff_angle_dist(2),min_diff_angle_dist(3));
    angle = acos(abs(axis_1(1) - center_lon)/sqrt((axis_1(1)-center_lon)*(axis_1(1)-center_lon) + (axis_1(2)-center_lat)*(axis_1(2)-center_lat)));
end


function res = axis_in_range(axis_1,axis_2,lon_size,lat_size)
    res = 1;
    if((axis_1(1)<=0)||(axis_1(2)<=0)||(axis_1(3)<=0)||(axis_1(4)<=0)||(axis_2(1)<=0)||(axis_2(2)<=0)||(axis_2(3)<=0)||(axis_2(4)<=0))
        res = 0;
        return;
    end
    if((axis_1(1)>lon_size)||(axis_1(2)>lat_size)||(axis_1(3)>lon_size)||(axis_1(4)>lat_size)||(axis_2(1)>lon_size)||(axis_2(2)>lat_size)||(axis_2(3)>lon_size)||(axis_2(4)>lat_size))
        res = 0;
        return;
    end
end


function ort_diff = ortogonal_diff(angle,lon,lat,center_lon,center_lat)
    new_angle = get_angle(lon - center_lon,lat - center_lat);
    ort_angle_1 = simplify_angle(new_angle + pi/2);
    ort_angle_2 = simplify_angle(new_angle - pi/2);
    ort_diff_1 = get_angle_diff(ort_angle_1,angle);
    ort_diff_2 = get_angle_diff(ort_angle_2,angle);
    if(ort_diff_1 < ort_diff_2)
        ort_diff = ort_diff_1;
    else 
        ort_diff = ort_diff_2;
    end
end


function [axis_1,axis_2] = get_axis(center_lon,center_lat,dist,num)
    if(num > dist + 1)
        axis_1(1) = 2*dist - num + 1 + center_lon;
        axis_1(2) = dist + center_lat;
    else
        axis_1(1) = dist + center_lon;
        axis_1(2) = num + center_lat - 1;
    end
    axis_1(3) = 2*center_lon - axis_1(1);
    axis_1(4) = 2*center_lat - axis_1(2);
    axis_2(1) = -axis_1(2) + center_lat + center_lon;
    axis_2(2) = axis_1(1) - center_lon + center_lat;
    axis_2(3) = axis_1(2) - center_lat + center_lon;
    axis_2(4) = -axis_1(1) + center_lon + center_lat;
end